/* NessusJ
 * Copyright (C) 1998 Alexis de Bernis
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This class is waiting for messages from nessusd. It reports them
 * to the window status (class NessusdStatus).
 *
 *  Please run class NessusJ instead.
 */

import java.lang.*;
import java.io.*;
import com.sun.java.swing.*;


class NessusdListener extends Thread
{
    DataInputStream fromNessusd;
    DataOutputStream forNessusd;
    boolean finished;
    NessusdStatus status;

    NessusdListener(NessusdStatus status,DataInputStream fromNessusd,
                                         DataOutputStream forNessusd)
    {
        super("NessusdListener");

        this.fromNessusd=fromNessusd;
        this.forNessusd=forNessusd;
        this.status=status;
        finished=false;

    }

    public void run()
    {
//*        System.out.println("NessusdListener launched");

        while(!finished)
        {
            waitForMessage();
        }
    }

    private void waitForMessage()
    {
        String received;

//*        System.out.println("Waiting for a message from server");

        try
        {
        received=fromNessusd.readLine();
        }catch(IOException e)
        {
            JOptionPane.showMessageDialog(status,"Error waiting for message","Error",JOptionPane.ERROR_MESSAGE);
            finished=true;
            return;
        }

        dispatchMessage(received);
    }

    private void dispatchMessage(String message)
    {
//*        System.out.println("Dispatching message : "+message);

        if(!isServerMessage(message))
        {
            JOptionPane.showMessageDialog(status,message+" :\nMessage is not from Server","Error",JOptionPane.ERROR_MESSAGE);
            return;
        }

        message=message.substring(11);         //Put out "SERVER <|> "

//*        System.out.println("message is now : "+message);

        if(message.substring(0,message.indexOf(" <|> ")).equals("HOLE"))
        {
            sendHoleMessage(message.substring(message.indexOf(" <|> ")+5));
            return;
        }

        if(message.substring(0,message.indexOf(" <|> ")).equals("INFO"))
        {
            sendInfoMessage(message.substring(message.indexOf(" <|> ")+5));
            return;
        }

        if(message.substring(0,message.indexOf(" <|> ")).equals("STAT"))
        {
            sendStatMessage(message.substring(message.indexOf(" <|> ")+5));
            return;
        }

        if(message.substring(0,message.indexOf(" <|> ")).equals("PORT"))
        {
            sendPortMessage(message.substring(message.indexOf(" <|> ")+5));
            return;
        }

        if(message.substring(0,message.indexOf(" <|> ")).equals("BYE"))
        {
            finished=true;
            status.jobFinished();
            return;
        }

        if(message.substring(0,message.indexOf(" <|> ")).equals("ERROR"))
        {
            String warning=message.substring(message.indexOf(" <|> ")+5);
            JOptionPane.showMessageDialog(status, warning.substring(0,warning.indexOf(" <|> ")),"Error",JOptionPane.ERROR_MESSAGE);
            return;
        }

        if(message.substring(0,message.indexOf(" <|> ")).equals("PING"))
        {
            sendPingMessage();
            return;
        }

        JOptionPane.showMessageDialog(status,"Error : Unknown message : "+message,"Error",JOptionPane.ERROR_MESSAGE);
        return;
        
    }


    private boolean isServerMessage(String message)
    {
        try
        {
            return(message.substring(0,message.indexOf(" <|> ")).equals("SERVER"));
        }catch(IndexOutOfBoundsException e)
        {
            System.out.println("Bad type for message : "+message+"\n");
            return(false);
        }
    }


    private void sendPortMessage(String args)
    {
        String host;
        int port;

        host=args.substring(0,args.indexOf(" <|> "));
        args=args.substring(args.indexOf(" <|> ")+5);
        port=Integer.parseInt(args.substring(0,args.indexOf(" <|> ")));

        status.postPort(host,port);
    }

    private void sendStatMessage(String args)
    {
        String host,info;

        host=args.substring(0,args.indexOf(" <|> "));
        args=args.substring(args.indexOf(" <|> ")+5);
        info=args.substring(0,args.indexOf(" <|> "));

        status.refreshHost(host,info);
    }


    private void sendHoleMessage(String args)
    {
        String host,description;
        int port;

//*        System.out.println("Launching hole message : "+args);

        try
        {
            host=args.substring(0,args.indexOf(" <|> "));
            args=args.substring(args.indexOf(" <|> ")+5);
            port=Integer.parseInt(args.substring(0,args.indexOf(':')));
            description=args.substring(args.indexOf(':')+1,args.indexOf(" <|> "));
            description=description.replace(';','\n');
        }catch(IndexOutOfBoundsException e)
        {
            System.out.println("Bad type of message : "+args+"\n");
            return;
        }

//*        System.out.println("Host : "+host);
//*        System.out.println("Port : "+port);
//*        System.out.println("Description : "+description+"\n");

        status.postHole(host,new Hole(port,description));
    }

    private void sendInfoMessage(String args)
    {
        String host,description;
        int port;

//*        System.out.println("Launching Info message : "+args);

        try
        {
            host=args.substring(0,args.indexOf(" <|> "));
            args=args.substring(args.indexOf(" <|> ")+5);
            port=Integer.parseInt(args.substring(0,args.indexOf(':')));
            description=args.substring(args.indexOf(':')+1,args.indexOf(" <|> "));
            description=description.replace(';','\n');
        }catch(IndexOutOfBoundsException e)
        {
            System.out.println("Bad type of message : "+args+"\n");
            return;
        }

//*        System.out.println("Host : "+host);
//*        System.out.println("Port : "+port);
//*        System.out.println("Description : "+description+"\n");

        status.postInfo(host,new Info(port,description));
    }

    private void sendPingMessage()
    {
        try
        {
            forNessusd.writeBytes("CLIENT <|> PONG <|> PING <|> CLIENT");
        } catch(IOException e)
        {
            return;
        }

    }
}


